@@ -9,6 +9,7 @@ class Nova(monasca_setup.detection.ServicePlugin):
 
     def __init__(self, template_dir, overwrite=True, args=None):
         service_params = {
+            'args': args,
             'template_dir': template_dir,
             'overwrite': overwrite,
             'service_name': 'compute',
@@ -21,11 +22,4 @@ class Nova(monasca_setup.detection.ServicePlugin):
             'search_pattern': '.*version=2.*'
         }
 
-        # Skip the http_check if disable_http_check is set
-        if args is not None:
-            args_dict = dict([a.split('=') for a in args.split()])
-            if args_dict.get('disable_http_check', default=False):
-                service_params['service_api_url'] = None
-                service_params['self.search_pattern'] = None
-
         super(Nova, self).__init__(service_params)
